import 'antd/dist/antd.css';
import React, { useContext, useEffect, useState } from 'react';

import { useUnauthRedirect } from '../../../hooks';
import { useRouter } from 'next/router';
import { Button, Table, Tag, Typography } from 'antd';
import { faFileLines, faUser } from '@fortawesome/free-solid-svg-icons';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { LoggedUserContext } from '../../../contexts/LoggedUserContext';
import { MainLayout } from '../../../layouts/MainLayout';
import AddDocumentModal from '../../../components/modals/AddDocumentModal';
import { AnnotationListInfo, DocumentListInfo, UserInfo } from '../../../api';
import { documentController, userController } from '../../../controllers';
import AssignDocumentModal from '../../../components/modals/AssignDocumentModal';
import { ShowToast } from '../../../utils/alerts';
import { TableDocInfo } from '../../../components/types/TableDocInfo';
import DocPreviewModal from '../../../components/modals/DocPreviewModal';

function AdminDocumentPage() {
    const redirecting = useUnauthRedirect('/login');
    const { logout, role } = useContext(LoggedUserContext);
    const [visibleAdd, setVisibleAdd] = React.useState(false);
    const [visibleAssign, setVisibleAssign] = React.useState(false);
    const [visiblePreview, setVisiblePreview] = React.useState(false);
    const router = useRouter();

    const [documents, setDocuments] = useState<TableDocInfo[]>([]);
    const [selectedDocs, setSelectedDocs] = useState<string[] | undefined[]>([]);
    const [previewDocContent, setPreviewDocContent] = useState<string>();
    const [previewDocName, setPreviewDocName] = useState<string>();

    async function fetchData() {
        const docs = (await documentController.documentsGet(0, 1000)).data.documents;

        // @ts-ignore
        const tableDocs: TableDocInfo[] = docs?.map((doc, index) => {
            return { key: index, ...doc };
        });

        if (!docs) {
            setDocuments([]);
        } else {
            setDocuments(tableDocs);
        }
    }

    useEffect(() => {
        if (!redirecting && role === 'ADMINISTRATOR') {
            fetchData();
        }
    }, [logout, redirecting, role, router]);

    const showAssignModal = () => {
        if (selectedDocs.length == 0) {
            ShowToast('Vyberte dokument pro přiřazení', 'warning', 3000, 'top-end');
        } else {
            setVisibleAssign(true);
        }
    };

    const showAddModal = () => {
        setVisibleAdd(true);
    };

    const showPreviewModal = async (id: string, name: string) => {
        const documentContent = (await documentController.documentDocumentIdGet(id)).data
            .content;
        setPreviewDocName(name);
        setPreviewDocContent(documentContent);
        setVisiblePreview(true);
    };

    const hideModal = () => {
        fetchData();
        setVisibleAdd(false);
        setVisibleAssign(false);
        setVisiblePreview(false);
    };

    const columns = [
        {
            title: 'Název dokumentu',
            dataIndex: 'name',
            key: 'name',
        },
        {
            title: 'Délka',
            dataIndex: 'length',
            key: 'length',
        },
        {
            title: 'Anotátoři',
            dataIndex: 'annotatingUsers',
            key: 'annotatingUsers',
            render: (columnData: UserInfo[], record: DocumentListInfo, index: number) => {
                return (
                    <div>
                        {columnData.map((e) => (
                            <span
                                key={e.username + '.' + record.id}
                                title={e.username ?? ''}
                            >
                                <FontAwesomeIcon
                                    icon={faUser}
                                    title={e.username ?? ''}
                                    className={'me-2'}
                                />
                            </span>
                        ))}
                    </div>
                );
            },
        },
        {
            title: '',
            key: 'action',
            dataIndex: ['id', 'name'],
            // @ts-ignore
            render: (text, row) => (
                <Button key={row.id} onClick={() => showPreviewModal(row.id, row.name)}>
                    Náhled
                </Button>
            ),
        },
    ];

    const rowSelection = {
        onChange: (selectedRowKeys: React.Key[], selectedRows: DocumentListInfo[]) => {
            // @ts-ignore
            setSelectedDocs(selectedRows.map((row) => row.id));
        },
    };

    return redirecting || role !== 'ADMINISTRATOR' ? null : (
        <MainLayout>
            <Typography.Title level={2}>
                <FontAwesomeIcon icon={faFileLines} /> Dokumenty
            </Typography.Title>
            <Button type={'primary'} onClick={showAddModal}>
                Nahrát dokument
            </Button>
            <Button onClick={showAssignModal}>Přiřadit dokumenty</Button>
            {visibleAdd && <AddDocumentModal onCancel={hideModal} />}
            {visibleAssign && (
                <AssignDocumentModal documentsIds={selectedDocs} onCancel={hideModal} />
            )}
            {visiblePreview && (
                <DocPreviewModal
                    onCancel={hideModal}
                    documentName={previewDocName}
                    content={previewDocContent}
                />
            )}

            <Table
                rowSelection={{
                    type: 'checkbox',
                    ...rowSelection,
                }}
                columns={columns}
                dataSource={documents}
                size="middle"
                scroll={{ y: 600 }}
            />
        </MainLayout>
    );
}

export default AdminDocumentPage;
